package de.hsbo.fbv.bmg.topology.networks;

/**
 * Network - Generisches Interface fr Knoten-Kanten-Netzwerke.<br>
 * Das Interface setzt voraus, dass Netzwerke dieses Objekttyps aus Knoten- und Kanten-Objekten 
 * mit festgelegten Objekttypen (Klassen) bestehen. Den Knoten und Kanten werden Schlsselobjekte 
 * zugewiesen, unter denen sie im Netwerk zugnglich sind; z.B. Punkte fr Knoten, Linien fr Kanten.
 * Um die Zuweisung von Objekten mit falschen Datentypen zu verhindern, knnen die Knoten- und 
 * Kantentypen festgelegt werden.<br>
 * Die Knoten und Kanten des Netwerks sollen nicht als eigenstndige Objekte vernderbar sein.
 * Die entsprechenden Interfaces Node und Edge verfgen daher nur ber Auskunfts-Methoden. Die 
 * Vernderung der netzwerkelemente erfolgt ausschlielich durch Methoden des Netzwerks.
 * Zum Anlegen von Knoten und Kanten stehen die Methoden addNode() und addEdge() zur Verfgung.
 * Gelscht werden Knoten und Kanten mit removeNode() und removeEdge().  <br>
 * 
 * @author Zimmermann, HS Bochum 2011<br>
 *
 * @param <NodeT> Datentyp des Knotenobjektes 
 * @param <EdgeT> Datentyp des Kantenobjektes
 */
public interface Network1T<NodeT,EdgeT> {

	/**
	 * Gibt die Anzahl der gespeicherten Knoten an.
	 * @return
	 */
	public int getNumNodes();
	
	/**
	 * Stellt fest, ob das angegebene Knotenobjekt im Netzwerk vorhanden ist.
	 * @param object
	 * @return
	 */
	public boolean containsNode(NodeT object);
	
	/**
	 * Holt den Knoten des angegebenen Knotenobjektes ab.
	 * @param object
	 * @return
	 */
	public Node<NodeT> getNode(NodeT object);
	
	/**
	 * Gibt alle Knoten des Netzwerks aus.
	 * @return
	 */
	public Node<NodeT>[] getNodes();
	
	/**
	 * Gibt die Anzahl der gespeicherten Kanten an.
	 * @return
	 */
	public int getNumEdges();
	
	/**
	 * Stellt fest, ob das angegebene Kantenobjekt im Netzwerk vorhanden ist.
	 * @param object
	 * @return
	 */
	public boolean containsEdge(EdgeT object);
	
	/**
	 * Holt die Kante des angegebenen Kantenobjektes ab.
	 * @param object
	 * @return
	 */	
	public Edge<EdgeT> getEdge(EdgeT object);
	
	/**
	 * Gibt alle Kanten des Netzwerks aus.
	 * @return
	 */	
	public Edge<EdgeT>[] getEdges();
	
	/**
	 * Fgt einen neuen Knoten ins Netzwerk ein. Jedes Knotenobjekt wird nur 
	 * einmal ins Netzwerk eingefgt.  
	 * @param object
	 * @return
	 */
	public boolean addNode(NodeT object);
	
	/**
	 * Fgt einen neuen Knoten zusammen mit einem Knotengewicht ins Netzwerk ein. 
	 * Jedes Knotenobjekt wird nur einmal ins Netzwerk eingefgt.  
	 * @param object
	 * @param weight
	 * @return
	 */
	public boolean addNode(NodeT object, double weight);
	
	/**
	 * Fgt eine neue Kante ins Netzwerk ein. Jedes Kantenobjekt wird nur 
	 * einmal ins Netzwerk eingefgt.  
	 * @param object
	 * @param start
	 * @param end
	 * @return
	 */	
	public boolean addEdge(EdgeT object, NodeT start, NodeT end);
	
	/**
	 * Fgt eine neue Kante zusammen mit einem Kantengewicht ins Netzwerk ein. 
	 * Jedes Kantenobjekt wird nur einmal ins Netzwerk eingefgt.  
	 * @param object
	 * @param start
	 * @param end
	 * @param weight
	 * @return
	 */
	public boolean addEdge(EdgeT object, NodeT start, NodeT end, double weight);	

	/**
	 * Entfernt einen isolierten Knoten aus dem Netzwerk.
	 * @param object
	 * @return
	 */
	public boolean removeNode(NodeT object);
	
	/**
	 * Entfernt eine Kante aus dem Netzwerk.
	 * @param object
	 * @return
	 */
	public boolean removeEdge(EdgeT object);
	
	/**
	 * Stellt fest ob das Netzwerk (noch) Knoten oder Kanten enthlt.
	 * @return
	 */
	public boolean isEmpty();

	/**
	 * Selektiert ein Teilnetz aus dem gegebenen Netzwerk. Das Ergebnis ist 
	 * implementierungsabhngig. Fr die Auswertung enthalten die Knoten und 
	 * Kanten einen Rckverweis auf das Vorgnger-Objekt. 
	 * Die Methode kann dazu verwendet werden, um nach dem Durchlauf eines 
	 * Bredth-First-Search- oder Depth-First-Search-Algorithmus den Ergebnis-
	 * Graphen zusammenszustellen. 
	 * @return
	 */
	public Network1T<NodeT,EdgeT> getSubnet();
	
	/**
	 * Selektiert ausgehend von einem Knoten ein Teilnetz aus dem gegebenen Netzwerk. 
	 * Das Ergebnis ist implementierungsabhngig. Fr die Auswertung enthalten die 
	 * Knoten und Kanten einen Rckverweis auf das Vorgnger-Objekt. 
	 * Die Methode ist verwendbar fr die Zusammenstellung des krzesten Weges zwischen 
	 * zwei Knoten.
	 * @param object
	 * @return
	 */
	public Network1T<NodeT,EdgeT> getSubnet(NodeT object);
	
	/**
	 * Selektiert ein Teilnetz ber alle Knoten bzw. Kanten, die den angegeben Zustand 
	 * bbesitzen. Das Ergebnis ist implementierungsabhngig. 
	 * @param state
	 * @return
	 */
	public Network1T<NodeT,EdgeT> getSubnet(int state);
	
	
	
}
