package de.hsbo.fbv.bmg.topology.networks;

/**
 * Network - Generisches Interface fr Maschen-Netzwerke entsprechend der DIME-Struktur.<br>
 * Das Interface setzt voraus, dass Netzwerke dieses Objekttyps aus Knoten-, Kanten- und 
 * Maschen-Objekten mit festgelegten Objekttypen (Klassen) bestehen. Den Netzwerkelementen 
 * werden Schlsselobjekte zugewiesen, unter denen sie im Netwerk zugnglich sind; z.B. 
 * Punkte fr Knoten, Linien fr Kanten und Flchen fr Kanten.
 * Um die Zuweisung von Objekten mit falschen Datentypen zu verhindern, knnen Knoten-, Kanten- 
 * und Maschentypen festgelegt werden.<br>
 * Die Netzwerkelemente sollen nicht als eigenstndige Objekte vernderbar sein.
 * Die entsprechenden Interfaces Node, Edge und Face verfgen daher nur ber Auskunfts-Methoden. Die 
 * Vernderung der Netzwerkelemente erfolgt ausschlielich durch Methoden des Netzwerks.
 * Zum Anlegen von Maschen steht die Methode addFace() zur Verfgung.
 * Gelscht werden Maschen mit removeFace().  <br>
 * 
 * @author Zimmermann, HS Bochum 2011<br>
 *
 * @param <NodeT> Datentyp des Knotenobjektes 
 * @param <EdgeT> Datentyp des Kantenobjektes
 * @param <FaceT> Datentyp des Maschenobjektes
 */

public interface Network2T<NodeT,EdgeT,FaceT> extends Network1T<NodeT,EdgeT> {
	
	
	/**
	 * Fgt eine neue Masche ins Netzwerk ein. Jedes Maschenobjekt wird nur 
	 * einmal ins Netzwerk eingefgt. Die angegebenen Kantenobjekte mssen 
	 * im Netzwerk vorhanden sein und einem geschlossenen Kantenzug bilden.   
	 * @param object
	 * @param edges
	 * @return
	 */	
	public boolean addFace(FaceT object, EdgeT[] edges);
	
	/**
	 * Fgt eine neue Masche mit einem Maschengewicht ins Netzwerk ein. 
	 * Jedes Maschenobjekt wird nur einmal ins Netzwerk eingefgt. Die 
	 * angegebenen Kantenobjekte mssen im Netzwerk vorhanden sein und 
	 * einen geschlossenen Kantenzug bilden.   
	 * @param object
	 * @param edges
	 * @param weight
	 * @return
	 */	
	public boolean addFace(FaceT object, EdgeT[] edges, double weight);
	
	/**
	 * Entfernt eine Masche aus dem Netzwerk.
	 * @param object
	 * @return
	 */
	public boolean removeFace(FaceT object);

	/**
	 * Stellt fest, ob das angegebene Maschenobjekt im Netzwerk vorhanden ist.
	 * @param object
	 * @return
	 */
	public boolean containsFace(FaceT object);
	
	/**
	 * Holt die Masche des angegebenen Maschenobjektes ab.
	 * @param object
	 * @return
	 */
	public Face<FaceT> getFace(FaceT object);
	
	/**
	 * Gibt alle Maschen des Netzwerks aus.
	 * @return
	 */
	public Face<FaceT>[] getFaces();
	
	/**
	 * Gibt die Anzahl der gespeicherten Maschen an.
	 * @return
	 */
	public int getNumFaces();
	

	
}
