package de.hsbo.fbv.bmg.topology.networks.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import de.hsbo.fbv.bmg.topology.networks.Edge;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.networks.Node;



/**
 * NetworkImpl - Adapterklasse von Network.
 * @author Zimmermann, HS Bochum 2011
 *
 * @param <NodeT> Knotentyp
 * @param <EdgeT> Kantentyp
 * 
 * @see Network1T
 */
public class SimpleNetwork<NodeT, EdgeT> implements Network1T<NodeT, EdgeT> {
	
	// Attribute
	Map<NodeT,Node<NodeT>> nodes = null;
	Map<EdgeT,Edge<EdgeT>> edges = null;
	
	/**
	 * Standard-Konstruktor
	 */
	public SimpleNetwork() {
		super();
		nodes = new HashMap<NodeT, Node<NodeT>>();
		edges = new HashMap<EdgeT, Edge<EdgeT>>();
	}
	
	/**
	 * Kopier-Konstruktor
	 * @param network
	 */
	public SimpleNetwork(Network1T<NodeT, EdgeT> network) {
		super();
		nodes = new HashMap<NodeT, Node<NodeT>>();
		edges = new HashMap<EdgeT, Edge<EdgeT>>();
		Node<NodeT>[] nnodes = network.getNodes(); 
		for (int j = 0; j < nnodes.length; j++) {
			NodeT nObj = nnodes[j].getObject();
			this.addNode(nObj);
			((SimpleNode<NodeT>)this.getNode(nObj)).setValues(nnodes[j]);
		}
		Edge<EdgeT>[] nedges = network.getEdges(); 
		for (int j = 0; j < nedges.length; j++) {
			EdgeT nObj = nedges[j].getObject();
			NodeT nObjStart = (NodeT)nedges[j].getStartNode().getObject();
			NodeT nObjEnd = (NodeT)nedges[j].getEndNode().getObject();
			this.addEdge(nObj, nObjStart, nObjEnd);
			((SimpleEdge<EdgeT>)this.getEdge(nObj)).setValues(nedges[j]);
		}
	}
	
	@Override
	public int getNumNodes() {
		return nodes.size();
	}

	@Override
	public Node<NodeT> getNode(NodeT object) {
		return nodes.get(object);
	}

	@Override
	public Node<NodeT>[] getNodes() {
		List<Node> list = new ArrayList<Node>();
		list.addAll(nodes.values());
		Node[] nodeArray = new Node[list.size()];
		return list.toArray(nodeArray);
	}

	@Override
	public int getNumEdges() {
		return edges.size();
	}

	@Override
	public Edge<EdgeT> getEdge(EdgeT object) {
		return edges.get(object);
	}

	@Override
	public Edge<EdgeT>[] getEdges() {
		List<Edge> list = new ArrayList<Edge>();
		list.addAll(edges.values());
		Edge[] edgeArray = new Edge[list.size()];
		return list.toArray(edgeArray);
	}

	@Override
	public synchronized boolean addNode(NodeT object, double weight) {
		if(this.containsNode(object))
			return false;
		else {
			Node node = new SimpleNode(object);
			node.setWeight(weight);
			nodes.put(object, node);
			return true;
		}
	}
	
	@Override
	public synchronized boolean addNode(NodeT object) {
		return addNode(object, 0.0);
	}

	@Override
	public synchronized boolean addEdge(EdgeT object, NodeT start, NodeT end, double weight) {
		if(this.containsEdge(object))
			return false;
		else {
			if(!this.containsNode(start))
				this.addNode(start);
			if(!this.containsNode(end))
				this.addNode(end);
			Node nstart = this.getNode(start);
			Node nend = this.getNode(end);
			Edge edge = new SimpleEdge(object, nstart, nend);
			edge.setWeight(weight);
			edges.put(object, edge);
			((SimpleNode)nstart).add(edge);
			((SimpleNode)nend).add(edge);
			return true;
		}
	}

	@Override
	public synchronized boolean addEdge(EdgeT object, NodeT start, NodeT end) {
		return addEdge(object, start, end, 1.0);
	}

	@Override
	public boolean removeNode(NodeT object) {
		if(this.containsNode(object)) {
			SimpleNode node = (SimpleNode)(nodes.get(object));
			if(node.isIsolated()) {
				this.nodes.remove(object);
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean removeEdge(EdgeT object) {
		if(this.containsEdge(object)) {
			this.nodes.remove(object);
			return true;
		}
		return false;
	}

	@Override
	public Network1T<NodeT, EdgeT> getSubnet(int state) {
		Network1T<NodeT, EdgeT> subnet = new SimpleNetwork<NodeT, EdgeT>();
		Edge[] edges = this.getEdges();
		for(int i=0; i<edges.length; i++) {
			if(edges[i].getState()==state) {
				subnet.addEdge((EdgeT)edges[i].getObject(),
						(NodeT)edges[i].getStartNode().getObject(),
						(NodeT)edges[i].getEndNode().getObject());
			}
		}
		return subnet;
	}

	@Override
	public boolean containsNode(NodeT object) {
		return nodes.containsKey(object);
	}

	@Override
	public boolean containsEdge(EdgeT object) {
		return edges.containsKey(object);
	}

	@Override
	public boolean isEmpty() {
		return (edges.isEmpty() && nodes.isEmpty());
	}

	@Override
	public String toString() {
		return "Network [\n\tnodes=" + nodes + "\n\tedges=" + edges + "\n]";
	}

	@Override
	public Network1T<NodeT, EdgeT> getSubnet() {
		Network1T<NodeT, EdgeT> result = new SimpleNetwork<NodeT, EdgeT>();
		Node<NodeT>[] nodes = this.getNodes();
		for(int i=0; i<nodes.length; i++) {
			if(nodes[i].getTrace()!=null) {
				result = addSubnet(result, nodes[i].getObject());
			}
		}
		return result;
	}

	@Override
	public Network1T<NodeT, EdgeT> getSubnet(NodeT object) {
		Network1T<NodeT, EdgeT> result = new SimpleNetwork<NodeT, EdgeT>();
		return addSubnet(result, object);
	}
	
	private Network1T<NodeT, EdgeT> addSubnet(Network1T<NodeT, EdgeT> result, NodeT object) {
		NodeT start = object; 
		Node<NodeT> nStart = this.getNode(start);
		result.addNode(nStart.getObject());
		result.getNode(nStart.getObject()).setWeight(nStart.getWeight());	
		do {
			if(nStart.getTrace()==nStart) {
				break;
			}
			Edge<EdgeT> edge = (Edge) nStart.getTrace();
			Node<NodeT> nEnd = edge.getOppositeNode(nStart);
			if (result.containsEdge(edge.getObject()))
				break;
			result.addEdge(edge.getObject(), nStart.getObject(), 
					nEnd.getObject());
			result.getEdge(edge.getObject()).setWeight(edge.getWeight());
			result.getNode(nEnd.getObject()).setWeight(nEnd.getWeight());
			nStart = nEnd;
		} while (true);
		return result;	
	}
	
	public Set<NodeT> getNodeObjects() {
		return this.nodes.keySet();
	}
	
	public Set<EdgeT> getEdgeObjects() {
		return this.edges.keySet();
	}
	
	/*
	public NodeT[] toNodeArray(NodeT[] array) {
		Set<NodeT> set = this.nodes.keySet();
		return set.toArray(array);
	}

	public EdgeT[] toEdgeArray(EdgeT[] array) {
		Set<EdgeT> set = this.edges.keySet();
		return set.toArray(array);
	}
	*/
}
